#version 330 core
struct Material 
{
	vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    float shininess;
};  

struct PointLight 
{
    vec3 position;
    
    float constant;
    float linear;
    float quadratic;
  
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;       
};

struct DirectionalLight 
{
	vec3 direction;

	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

struct SpotLight 
{
    vec3 position;
    vec3 direction;
    float innerCutOff;
    float outerCutOff;
  
    float constant;
    float linear;
    float quadratic;
  
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;       
};

in vec3 FragPos;  
in vec3 Normal;  
in vec2 TexCoords;
  
out vec4 color;
  
uniform vec3 cameraPos;
uniform Material material;
uniform PointLight pointLight;
uniform DirectionalLight directionalLight;
uniform SpotLight spotLight;
uniform SpotLight flashLight;

uniform sampler2D texture_diffuse1;
// how many percent of the calculation will depend on the texture. the remaining will depend on the material
uniform float texturePercentage;


//for shadows and projection
uniform sampler2DShadow dirShadowMap;
uniform sampler2DShadow spotShadowMap;
uniform sampler2DShadow projShadowMap; 
uniform sampler2D projTexture;
in vec4 DirShadowCoord;
in vec4 SpotShadowCoord;
in vec4 ProjShadowCoord;
uniform float spotlight_bool;
uniform float projector_bool;
uniform float pointlight_bool;
float alphaLight(vec4 shadowCoord, sampler2DShadow shadowMap);


vec3 calculateDirectionalLight(DirectionalLight light, vec3 normal, vec3 viewDir);
vec3 calculatePointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir);
vec3 calculateSpotLight(SpotLight light, vec3 normal, vec3 fragPos, vec3 viewDir);

vec3 calculateProjectedTexture(sampler2DShadow pshadow, sampler2D projTexture, vec4 ProjShadowCoord, vec3 result);


void main()
{
    vec3 norm = normalize(Normal);
	vec3 viewDir = normalize(cameraPos - FragPos);

	vec3 result = vec3(0,0,0);
	
	result += calculateSpotLight(flashLight, norm, FragPos, viewDir);
	result += calculateDirectionalLight(directionalLight, norm, viewDir) * alphaLight(DirShadowCoord, dirShadowMap);	
		
	if(pointlight_bool > 0.5){
		result += calculatePointLight(pointLight, norm, FragPos, viewDir);
	}

	if(spotlight_bool > 0.5){
		result += calculateSpotLight(spotLight, norm, FragPos, viewDir) * alphaLight(SpotShadowCoord, spotShadowMap);
	}

	if(projector_bool > 0.5){
		result = calculateProjectedTexture(projShadowMap, projTexture, ProjShadowCoord, result);
	}

    color = vec4(result, 1.0);
} 

vec3 calculateDirectionalLight(DirectionalLight light, vec3 normal, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    float diff = max(dot(normal, lightDir), 0.0);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
	vec3 texturePixel = vec3(texture(texture_diffuse1, TexCoords));
	float materialPercentage = 1-texturePercentage;
    vec3 ambient = light.ambient * (materialPercentage * material.ambient + texturePercentage * texturePixel);
    vec3 diffuse = light.diffuse * diff * (materialPercentage * material.diffuse + texturePercentage * texturePixel);
    vec3 specular = light.specular * spec * (materialPercentage * material.specular + texturePercentage * texturePixel);
    return (ambient + diffuse + specular);
}

vec3 calculatePointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir)
{
    vec3 lightDir = normalize(light.position - fragPos);
    float diff = max(dot(normal, lightDir), 0.0);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    float distance = length(light.position - fragPos);
    float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));
	vec3 texturePixel = vec3(texture(texture_diffuse1, TexCoords));
	float materialPercentage = 1-texturePercentage;
    vec3 ambient = light.ambient * (materialPercentage * material.ambient + texturePercentage * texturePixel) * attenuation;
    vec3 diffuse = light.diffuse * diff * (materialPercentage * material.diffuse + texturePercentage * texturePixel) * attenuation;
    vec3 specular = light.specular * spec * (materialPercentage * material.specular + texturePercentage * texturePixel) * attenuation;
    return (ambient + diffuse + specular);
}

vec3 calculateSpotLight(SpotLight light, vec3 normal, vec3 fragPos, vec3 viewDir)
{
    vec3 lightDir = normalize(light.position - fragPos);
    float diff = max(dot(normal, lightDir), 0.0);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    float distance = length(light.position - fragPos);
    float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));
    float theta = dot(lightDir, normalize(-light.direction)); 
    float epsilon = light.innerCutOff - light.outerCutOff;
    float intensity = clamp((theta - light.outerCutOff) / epsilon, 0.0, 1.0);
	vec3 texturePixel = vec3(texture(texture_diffuse1, TexCoords));
	float materialPercentage = 1-texturePercentage;
    vec3 ambient = light.ambient * (materialPercentage * material.ambient + texturePercentage * texturePixel) * attenuation * intensity;
    vec3 diffuse = light.diffuse * diff * (materialPercentage * material.diffuse + texturePercentage * texturePixel) * attenuation * intensity;
    vec3 specular = light.specular * spec * (materialPercentage * material.specular + texturePercentage * texturePixel) * attenuation * intensity;
    return (ambient + diffuse + specular);
}


float alphaLight(vec4 shadowCoord, sampler2DShadow shadowMap)
{
	float vis = 1.0f;
	
	if( shadowCoord.w > 0){
		vis = textureProj ( shadowMap , shadowCoord );
		if(vis < 0.2) vis = 0.2;
	}

	return vis;
}

vec3 calculateProjectedTexture(sampler2DShadow shadow, sampler2D projtexture, vec4 coord, vec3 result)
{
	if( coord.w > 0){
		if(textureProj ( shadow , coord ) == 1)
		{
			result = result + vec3(texture2DProj(projtexture, coord)) * 0.4;
		}
	}

	return result;
}