#version 330 core
struct Material 
{
	vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    float shininess;
};  

struct PointLight 
{
    vec3 position;
    
    float constant;
    float linear;
    float quadratic;
  
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;       
};

struct DirectionalLight 
{
	vec3 direction;

	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

struct SpotLight 
{
    vec3 position;
    vec3 direction;
    float innerCutOff;
    float outerCutOff;
  
    float constant;
    float linear;
    float quadratic;
  
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;       
};

in vec3 FragPos;  
in vec3 Normal;  
in vec2 TexCoords;
  
out vec4 color;
  
uniform vec3 cameraPos;
uniform Material material;
uniform PointLight pointLight;
uniform DirectionalLight directionalLight;
uniform SpotLight spotLight;
uniform SpotLight flashLight;

uniform sampler2D texture_diffuse1;

// how many percent of the calculation will depend on the texture. the remaining will depend on the material
uniform float texturePercentage;
uniform float numberOfShades;

vec3 calculateDirectionalLight(DirectionalLight light, vec3 normal, vec3 viewDir);
vec3 calculatePointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir);
vec3 calculateSpotLight(SpotLight light, vec3 normal, vec3 fragPos, vec3 viewDir);
uniform float spotlight_bool;
uniform float pointlight_bool;

void main()
{
    vec3 norm = normalize(Normal);
	vec3 viewDir = normalize(cameraPos - FragPos);

	vec3 c = calculateDirectionalLight(directionalLight, norm, viewDir);
	
	if(pointlight_bool > 0.5){
		c += calculatePointLight(pointLight, norm, FragPos, viewDir);
	}
	
	c += calculateSpotLight(flashLight, norm, FragPos, viewDir);
	
	if(spotlight_bool > 0.5){
		c += calculateSpotLight(spotLight, norm, FragPos, viewDir);
	}
	
	vec3 texturePixel = vec3(texture(texture_diffuse1, TexCoords));
    color = vec4(c * (material.diffuse * (1 - texturePercentage) + texturePixel * texturePercentage), 1.0);

} 

vec3 calculateDirectionalLight(DirectionalLight light, vec3 normal, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    float diff = max(dot(normal, lightDir), 0.0);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    float intensity = 0.1 + diff + spec;
	intensity = ceil(intensity * numberOfShades) / numberOfShades;
	return light.diffuse * intensity;
}

vec3 calculatePointLight(PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir)
{
    vec3 lightDir = normalize(light.position - fragPos);
    float diff = max(dot(normal, lightDir), 0.0);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    float distance = length(light.position - fragPos);
    float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));
    float intensity = (0.1 + diff + spec) * attenuation;
	intensity = ceil(intensity * numberOfShades) / numberOfShades;
	return light.diffuse * intensity;
}

vec3 calculateSpotLight(SpotLight light, vec3 normal, vec3 fragPos, vec3 viewDir)
{
    vec3 lightDir = normalize(light.position - fragPos);
    float diff = max(dot(normal, lightDir), 0.0);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    float distance = length(light.position - fragPos);
    float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));
    float theta = dot(lightDir, normalize(-light.direction)); 
    float epsilon = light.innerCutOff - light.outerCutOff;
    float intensity = clamp((theta - light.outerCutOff) / epsilon, 0.0, 1.0);
    intensity = (0.1 + diff + spec) * attenuation * intensity;
	intensity = ceil(intensity * numberOfShades) / numberOfShades;
	return light.diffuse * intensity;
}